"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpServer = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const swagger_1 = require("@nestjs/swagger");
const http_filter_1 = require("../../http/nest/http.filter");
const http_module_1 = require("../../http/nest/http.module");
class HttpServer {
    constructor(client, version) {
        this.client = client;
        this.version = version;
    }
    async listen(host, port) {
        const app = await this.init();
        await app.listen(port, host);
        return app.getUrl();
    }
    async init() {
        const app = await core_1.NestFactory.create(http_module_1.HttpModule.forRoot(this.client), {
            logger: false
        });
        app.useGlobalPipes(new common_1.ValidationPipe({ transform: true }));
        app.useGlobalFilters(new http_filter_1.HttpFilter());
        app.enableCors();
        const swaggerConfig = new swagger_1.DocumentBuilder()
            .setTitle('iola')
            .setDescription('API documentation for iola client')
            .setVersion(this.version)
            .build();
        const swaggerDocument = swagger_1.SwaggerModule
            .createDocument(app, swaggerConfig);
        swagger_1.SwaggerModule
            .setup('swagger', app, swaggerDocument);
        return app;
    }
}
exports.HttpServer = HttpServer;
