"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageUtil = void 0;
const util_1 = require("util");
const common_1 = require("../../common");
class MessageUtil {
    static packToString(message) {
        const data = typeof message === 'string'
            ? message
            : JSON.stringify(message) + '';
        const format = typeof message === 'object' && message !== null
            ? common_1.MessageFormat.JSON
            : common_1.MessageFormat.String;
        return { format, data };
    }
    static packToBuffer(message) {
        const info = {
            format: common_1.MessageFormat.ByteArray,
            data: Buffer.from(message)
        };
        return info;
    }
    static unpack(message) {
        const info = {
            format: common_1.MessageFormat.String,
            data: ''
        };
        if (typeof message === 'string') {
            if (message.includes('{') || message.includes('[')) {
                try {
                    info.data = JSON.parse(message);
                    info.format = common_1.MessageFormat.JSON;
                }
                catch (error) { }
            }
            else {
                info.data = message;
                info.format = common_1.MessageFormat.String;
            }
        }
        else {
            info.format = common_1.MessageFormat.ByteArray;
            info.data = Array.from(new Uint8Array(message));
        }
        return info;
    }
    static parse(message) {
        const info = {
            format: common_1.MessageFormat.String,
            data: ''
        };
        if (typeof message === 'string') {
            info.data = message;
            info.format = common_1.MessageFormat.String;
        }
        if (typeof message === 'number') {
            info.data = message;
            info.format = common_1.MessageFormat.Number;
        }
        if (typeof message === 'boolean') {
            info.data = message;
            info.format = common_1.MessageFormat.Boolean;
        }
        if (typeof message === 'object' && message !== null) {
            info.data = message;
            info.format = common_1.MessageFormat.JSON;
        }
        if (Buffer.isBuffer(message)) {
            info.data = Array.from(new Uint8Array(message));
            info.format = common_1.MessageFormat.ByteArray;
        }
        if (message === null) {
            info.data = message;
            info.format = common_1.MessageFormat.Null;
        }
        return info;
    }
    static humanize(message, options) {
        if (typeof message === 'string') {
            return message;
        }
        return (0, util_1.inspect)(message, {
            colors: true,
            compact: true,
            maxArrayLength: options?.maxArrayLength || 16,
            maxStringLength: options?.maxStringLength || 50,
            depth: 2,
        });
    }
    static findRequestId(message) {
        const requestIdKeys = [
            'requestid',
            'request_id',
            'reqid',
            'req_id',
            'traceid',
            'trace_id',
        ];
        let requestIdKey;
        if (typeof message === 'object' && message !== null && !Array.isArray(message)) {
            Object.keys(message).forEach(key => {
                if (requestIdKeys.includes(key.toLowerCase())) {
                    requestIdKey = key;
                }
            });
        }
        return requestIdKey
            ? { key: requestIdKey, value: message[requestIdKey] }
            : undefined;
    }
}
exports.MessageUtil = MessageUtil;
