"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocketFactory = void 0;
const socket_1 = require("../../socket");
const net_socket_client_1 = require("../../socket/client/net-socket.client");
const net_socket_sync_client_1 = require("../../socket/client/net-socket.sync.client");
const socketio_client_1 = require("../../socket/client/socketio.client");
const websocket_client_1 = require("../../socket/client/websocket.client");
const binary_message_store_1 = require("../../socket/store/binary-message.store");
const event_store_1 = require("../../socket/store/event.store");
class SocketFactory {
    static createClient(options) {
        const factory = {
            [socket_1.SocketType.SocketIO]: () => new socketio_client_1.SocketIOClient(options, new event_store_1.EventStore()),
            [socket_1.SocketType.WebSocket]: () => new websocket_client_1.WebSocketClient(options, new event_store_1.EventStore()),
            [socket_1.SocketType.Tcp]: () => this.createNetSocket(options),
            [socket_1.SocketType.Unix]: () => this.createNetSocket(options),
        };
        return factory[options.type]();
    }
    static createNetSocket(options) {
        if (options.netSync) {
            return new net_socket_sync_client_1.NetSocketSyncClient(options, new event_store_1.EventStore());
        }
        return new net_socket_client_1.NetSocketClient(options, new event_store_1.EventStore(), new binary_message_store_1.BinaryMessageStore());
    }
}
exports.SocketFactory = SocketFactory;
