"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BinaryMessageStore = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
class BinaryMessageStore {
    constructor() {
        this.store = [];
        this.ticker$ = (0, rxjs_1.interval)(2000);
        this.loading = false;
    }
    group() {
        const diffBetweenMessages = 500;
        return this.ticker$.pipe((0, operators_1.map)(() => this.groupMessage(diffBetweenMessages)), (0, operators_1.filter)(buffer => buffer.length > 0));
    }
    add(data) {
        this.store.push({
            date: Date.now(),
            data: data
        });
    }
    groupMessage(diffBetweenMessages) {
        let buffer = Buffer.alloc(0);
        if (!this.loading) {
            if (this.store.length) {
                this.loading = true;
                const count = this.countGroupedChunks(diffBetweenMessages);
                if (count > 0) {
                    for (let i = 0; i < count; i++) {
                        const message = this.store.shift();
                        const data = message?.data || Buffer.alloc(0);
                        buffer = Buffer.concat([buffer, data]);
                    }
                }
                this.loading = false;
            }
        }
        return buffer;
    }
    countGroupedChunks(diffBetweenMessages) {
        let count = 0;
        let lastDate = 0;
        for (const message of this.store) {
            if (lastDate === 0) {
                count++;
                lastDate = message.date;
                continue;
            }
            if (message.date - lastDate >= diffBetweenMessages) {
                break;
            }
            else {
                count++;
                lastDate = message.date;
            }
        }
        return count;
    }
}
exports.BinaryMessageStore = BinaryMessageStore;
