#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const cli_1 = require("./api/cli");
const http_1 = require("./api/http");
const socket_1 = require("./core/socket");
(async () => {
    try {
        const version = '0.5.4';
        const config = cli_1.CliFactory
            .createParser(version)
            .parse();
        detectSwaggerUIAssets();
        const client = socket_1.SocketFactory.createClient({
            type: config.socketType,
            address: config.socketAddress,
            binaryEncoding: config.binaryEncoding,
            connectionTimeout: config.connectionTimeout,
            reconnectionInterval: config.reconnectionInterval,
            headers: config.headers,
            ioAuth: config.ioAuth,
            ioTransport: config.ioTransport,
            netSync: config.netSync,
            replyTimeout: config.replyTimeout,
        });
        process.on('uncaughtException', error => client.store.add({
            type: socket_1.SocketEventType.Error,
            date: new Date(),
            message: {
                uncaughtException: error.message
            },
        }));
        process.on('unhandledRejection', error => client.store.add({
            type: socket_1.SocketEventType.Error,
            date: new Date(),
            message: {
                uncaughtException: error
            },
        }));
        const server = http_1.HttpFactory
            .createServer(client, version);
        await cli_1.CliFactory
            .createInteractive(config)
            .listen(server, client);
    }
    catch (error) {
        const message = error instanceof Error
            ? error.message
            : JSON.stringify(error);
        console.error(message);
        process.exit(1);
    }
})();
function detectSwaggerUIAssets() {
    path.join(__dirname, '../node_modules/swagger-ui-dist/swagger-ui.css');
    path.join(__dirname, '../node_modules/swagger-ui-dist/favicon-16x16.png');
    path.join(__dirname, '../node_modules/swagger-ui-dist/favicon-32x32.png');
}
